package cn.com.qmth.examcloud.core.oe.student.service.impl;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import cn.com.qmth.examcloud.core.oe.student.base.bean.ExamQuestion;
import cn.com.qmth.examcloud.core.oe.student.base.bean.ExamRecordQuestions;
import cn.com.qmth.examcloud.core.oe.student.service.ExamRecordQuestionsService;
import cn.com.qmth.examcloud.question.commons.core.paper.DefaultPaper;
import cn.com.qmth.examcloud.question.commons.core.paper.DefaultQuestionGroup;
import cn.com.qmth.examcloud.question.commons.core.paper.DefaultQuestionStructureWrapper;
import cn.com.qmth.examcloud.question.commons.core.paper.DefaultQuestionUnitWrapper;
import cn.com.qmth.examcloud.support.redis.RedisKeyHelper;
import cn.com.qmth.examcloud.web.redis.RedisClient;

@Service("examRecordQuestionsService")
public class ExamRecordQuestionsServiceImpl implements ExamRecordQuestionsService {

    @Autowired
    private RedisClient redisClient;

    @Override
    public void saveExamQuestion(Long examRecordDataId,Integer order, ExamQuestion question) {
        String key = RedisKeyHelper.getBuilder().studentAnswerKey(examRecordDataId, order);
        redisClient.set(key + examRecordDataId, question,-1);
    }

    @Override
    public ExamQuestion getExamQuestion(Long examRecordDataId,Integer order) {
        String key = RedisKeyHelper.getBuilder().studentAnswerKey(examRecordDataId, order);
        return redisClient.get(key + examRecordDataId, ExamQuestion.class);
    }

    @Override
    public void deleteExamQuestion(Long examRecordDataId,Integer order) {
        String key = RedisKeyHelper.getBuilder().studentAnswerKey(examRecordDataId, order);
        redisClient.delete(key + examRecordDataId);
    }

    @Override
    public ExamRecordQuestions createExamRecordQuestions(Long examRecordDataId, DefaultPaper defaultPaper) {
        ExamRecordQuestions examRecordQuestions = new ExamRecordQuestions();

        List<ExamQuestion> examQuestionEntityList = new ArrayList<ExamQuestion>();
        List<DefaultQuestionGroup> defaultQuestionGroups = defaultPaper.getQuestionGroupList();
        int order = 0;
        for (int i = 0; i < defaultQuestionGroups.size(); i++) {
            DefaultQuestionGroup defaultQuestionGroup = defaultQuestionGroups.get(i);
            List<DefaultQuestionStructureWrapper> defaultQuestionStructureWrappers = defaultQuestionGroup
                    .getQuestionWrapperList();
            for (DefaultQuestionStructureWrapper defaultQuestionStructureWrapper : defaultQuestionStructureWrappers) {
                List<DefaultQuestionUnitWrapper> questionUnitWrapperList = defaultQuestionStructureWrapper
                        .getQuestionUnitWrapperList();
                for (DefaultQuestionUnitWrapper defaultQuestionUnitWrapper : questionUnitWrapperList) {
                    ExamQuestion examQuestionEntity = new ExamQuestion();
                    examQuestionEntity.setExamRecordDataId(examRecordDataId);
                    examQuestionEntity.setMainNumber(i + 1);
                    examQuestionEntity.setOrder(++order);
                    examQuestionEntity.setQuestionId(defaultQuestionStructureWrapper.getQuestionId());
                    examQuestionEntity.setQuestionScore(defaultQuestionUnitWrapper.getQuestionScore());
                    examQuestionEntity.setQuestionType(defaultQuestionUnitWrapper.getQuestionType());
                    examQuestionEntity.setOptionPermutation(defaultQuestionUnitWrapper.getOptionPermutation());
                    examQuestionEntity.setAudioPlayTimes(null);
                    examQuestionEntity.setAnswerType(defaultQuestionUnitWrapper.getAnswerType());

                    examQuestionEntityList.add(examQuestionEntity);
                }
            }
        }

        examRecordQuestions.setExamQuestions(examQuestionEntityList);
        examRecordQuestions.setExamRecordDataId(examRecordDataId);
        examRecordQuestions.setCreationTime(new Date());
        for(ExamQuestion eq:examRecordQuestions.getExamQuestions()) {
            saveExamQuestion(eq.getExamRecordDataId(), eq.getOrder(), eq);
        }
        return examRecordQuestions;
    }

}
